/* 
Library for Sparkfun Si4703 breakout board.
Simon Monk. 2011-09-09

This is a library wrapper and a few extras to the excellent code produced
by Nathan Seidle from Sparkfun (Beerware).

Nathan's comments......

Look for serial output at 57600bps.

The Si4703 ACKs the first byte, and NACKs the 2nd byte of a read.

1/18 - after much hacking, I suggest NEVER write to a register without first reading the contents of a chip.
ie, don't updateRegisters without first readRegisters.

If anyone manages to get this datasheet downloaded
http://wenku.baidu.com/view/d6f0e6ee5ef7ba0d4a733b61.html
Please let us know. It seem to be the latest version of the programming guide. It had a change on page 12 (write 0x8100 to 0x07)
that allowed me to get the chip working..

Also, if you happen to find "AN243: Using RDS/RBDS with the Si4701/03", please share. I love it when companies refer to
documents that don't exist.

1/20 - Picking up FM stations from a plane flying over Portugal! Sweet! 93.9MHz sounds a little soft for my tastes,s but
it's in Porteguese.

ToDo:
Display current status (from 0x0A) - done 1/20/11
Add RDS decoding - works, sort of
Volume Up/Down - done 1/20/11
Mute toggle - done 1/20/11
Tune Up/Down - done 1/20/11
Read current channel (0xB0) - done 1/20/11
Setup for Europe - done 1/20/11
Seek up/down - done 1/25/11

The Si4703 breakout does work with line out unsigned shorto a stereo or other amplifier. Be sure to test with different length 3.5mm
cables. Too short of a cable may degrade reception.
*/

#ifndef SparkFunSi4703_h
#define SparkFunSi4703_h

#include "Arduino.h"
#include "Wire.h"

#define Si4703 0x10                                 //0b._001.0000 I2C address of Si4703 - note that the Wire function assumes non-left-shifted I2C address, not 0b.0010.000W
 
//Define the register names
#define DEVICEID 0x00
#define CHIPID 0x01
#define POWERCFG 0x02
#define CHANNEL 0x03
#define SYSCONFIG1 0x04
#define SYSCONFIG2 0x05
#define SYSCONFIG3 0x06
#define TEST1 0x07
#define STATUSRSSI 0x0A
#define READCHAN 0x0B
#define RDSA 0x0C
#define RDSB 0x0D
#define RDSC 0x0E
#define RDSD 0x0F
 
class Si4703_Breakout
{
 public:
  Si4703_Breakout(unsigned char resetPin, unsigned char sdioPin, unsigned char sclkPin);
  void PowerOn();
  void SetFrequency(unsigned short Freq);
 	unsigned short GetFrequency();
 	unsigned short Seek(unsigned char SeekDirection);
  void SetVolume(unsigned char Volume);
  unsigned short SeekUp();
  unsigned short SeekDown();
 	void ReadRDS(unsigned char* Message, unsigned long Timeout);
  void GetStatus(unsigned char* RSSI, unsigned char* IsStereo, unsigned char* IsRDSR);
  
 private:
  unsigned short _resetPin;
 	unsigned short _sdioPin;
 	unsigned short _sclkPin;
 	void Si4703_Init();
 	void ReadRegisters();
 	unsigned char UpdateRegisters();
 	bool IsPoweredOn = false;
 	unsigned short Si4703_Registers[16];              //There are 16 registers, each 16 bits large
 	static const unsigned char FAIL = 0;
 	static const unsigned char SUCCESS = 1;
  static const unsigned char SEEK_DOWN = 0;         //Direction used for seeking
  static const unsigned char SEEK_UP = 1;
};

#endif
